@@ -138,7 +138,7 @@ void nyx_susp_fetch(register nyx_susp_type susp, snd_list_type snd_list)
       n = susp->len - susp->susp.current;
 
    err = susp->callback(out_ptr, susp->channel,
-                            susp->susp.current, n, susp->userdata);
+                            susp->susp.current, n, 0, susp->userdata);
    if (err)
       longjmp(nyx_cntxt.c_jmpbuf, 1);      
 
@@ -185,6 +185,12 @@ void nyx_get_captured_output(int *out_len,
    nyx_output_pos = 0;
 }
 
+void nyx_set_audio_params( double rate )
+{
+   /* Bind the sample rate to the "*sound-srate*" global */
+   setvalue(xlenter("*SOUND-SRATE*"), cvflonum(rate));
+}
+
 void nyx_set_input_audio(nyx_audio_callback callback,
                          void *userdata,
                          int num_channels,
@@ -262,20 +268,38 @@ int is_labels(LVAL expr)
    /* make sure that we have a list whose first element is a
       list of the form (time "label") */
 
+   LVAL label;
+   LVAL first;
+   LVAL second;
+   LVAL third;
+
    if (!consp(expr))
       return 0;
 
-   if (!consp(car(expr)))
+   label = car(expr);
+
+   if (!consp(label))
       return 0;
 
-   if (!(floatp(car(car(expr))) || fixp(car(car(expr)))))
+   first = car(label);
+   if (!(floatp(first) || fixp(first)))
       return 0;
 
-   if (!consp(cdr(car(expr))))
+   if (!consp(cdr(label)))
       return 0;
 
-   if (!(stringp(car(cdr(car(expr))))))
-      return 0;
+   second = car(cdr(label));
+
+   if (floatp(second) || fixp(second)) {
+      if (!consp(cdr(cdr(label))))
+         return 0;
+      third = car(cdr(cdr(label)));
+      if (!(stringp(third)))
+         return 0;
+   }
+   else
+      if (!(stringp(second)))
+         return 0;
 
    /* If this is the end of the list, we're done */
 
@@ -385,7 +409,7 @@ int nyx_get_audio_num_channels()
       return 1;
 }
 
-void nyx_get_audio(nyx_audio_callback callback, void *userdata)
+int nyx_get_audio(nyx_audio_callback callback, void *userdata)
 {
    sample_block_type block;
    sound_type snd;
@@ -393,10 +417,12 @@ void nyx_get_audio(nyx_audio_callback callback, void *userdata)
    float *buffer;
    long bufferlen;
    long *totals;
+   long *lens;
    long cnt;
    int result = 0;
    int num_channels;
    int ch, i;
+   int success = FALSE;
 
    if (nyx_get_type(nyx_result) != nyx_audio)
       return;
@@ -404,6 +430,7 @@ void nyx_get_audio(nyx_audio_callback callback, void *userdata)
    num_channels = nyx_get_audio_num_channels();
    snds = (sound_type *)malloc(num_channels * sizeof(sound_type));
    totals = (long *)malloc(num_channels * sizeof(long));
+   lens = (long *)malloc(num_channels * sizeof(long));
 
    /* setup the error return */
    xlbegin(&nyx_cntxt,CF_TOPLEVEL|CF_CLEANUP|CF_BRKLEVEL,(LVAL)1);
@@ -417,6 +444,7 @@ void nyx_get_audio(nyx_audio_callback callback, void *userdata)
          snd = getsound(getelement(nyx_result, ch));
       snds[ch] = snd;
       totals[ch] = 0;
+      lens[ch] = snd_length(snd, snd->stop);
    }
 
    buffer = NULL;
@@ -448,20 +476,28 @@ void nyx_get_audio(nyx_audio_callback callback, void *userdata)
             buffer[i] *= snd->scale;
 
          result = callback(buffer, ch,
-                           totals[ch], cnt, userdata);
+                           totals[ch], cnt, lens[ch], userdata);
 
-         if (result == 0)
-            totals[ch] += cnt;
+         if (result != 0) {
+            goto finish;
+         }
+
+         totals[ch] += cnt;
       }
    }
 
+   success = TRUE;
+
  finish:
-      if (buffer)
-         free(buffer);
+   if (buffer)
+      free(buffer);
    free(snds);
    free(totals);
+   free(lens);
 
    xlend(&nyx_cntxt);
+
+   return success;
 }
 
 int nyx_get_int()
@@ -506,11 +542,15 @@ int nyx_get_num_labels()
 }
 
 void nyx_get_label(int index,
-                   double *time,
+                   double *start_time,
+                   double *end_time,
                    const char **label)
 {
    LVAL s = nyx_result;
-   LVAL t_expr;
+   LVAL label_expr;
+   LVAL t0_expr;
+   LVAL t1_expr;
+   LVAL str_expr;
 
    if (nyx_get_type(nyx_result) != nyx_labels)
       return;
@@ -523,12 +563,29 @@ void nyx_get_label(int index,
       s = cdr(s);
    }
 
-   t_expr = car(car(s));
-   if (floatp(t_expr))
-      *time = getflonum(t_expr);
-   else if (fixp(t_expr))
-      *time = (double)getfixnum(t_expr);
-   *label = (const char *)getstring(car(cdr(car(s))));
+   /* We either have (t0 "label") or (t0 t1 "label") */
+
+   label_expr = car(s);
+   t0_expr = car(label_expr);
+   t1_expr = car(cdr(label_expr));
+   if (stringp(t1_expr)) {
+      str_expr = t1_expr;
+      t1_expr = t0_expr;
+   }
+   else
+      str_expr = car(cdr(cdr(label_expr)));
+
+   if (floatp(t0_expr))
+      *start_time = getflonum(t0_expr);
+   else if (fixp(t0_expr))
+      *start_time = (double)getfixnum(t0_expr);
+
+   if (floatp(t1_expr))
+      *end_time = getflonum(t1_expr);
+   else if (fixp(t1_expr))
+      *end_time = (double)getfixnum(t1_expr);
+
+   *label = (const char *)getstring(str_expr);
 }
 
 const char *nyx_get_error_str()
