@@ -6,13 +6,39 @@
 
   Dominic Mazzoni
 
-**********************************************************************/
+*******************************************************************//**
 
-#include "FreqWindow.h"
+\class FreqWindow
+\brief Displays a spectrum plot of the waveform.  Has options for
+selecting parameters of the plot.
+
+Has a feature that finds peaks and reports their value as you move
+the mouse around.
+
+*//****************************************************************//**
+
+\class FreqPlot
+\brief Works with FreqWindow to dsplay a spectrum plot of the waveform.
+This class actually does the graph display.
+
+Has a feature that finds peaks and reports their value as you move
+the mouse around.
+
+*//*******************************************************************/
+
+/*
+  Salvo Ventura - November 2006
+  Extended range check for additional FFT windows
+*/
+
+
+#include "Audacity.h"
 
 // For compilers that support precompilation, includes "wx/wx.h".
 #include <wx/wxprec.h>
 
+
+
 #ifdef __BORLANDC__
 #pragma hdrstop
 #endif
@@ -27,22 +53,29 @@
 #include <wx/file.h>
 #include <wx/filedlg.h>
 #include <wx/intl.h>
+#include <wx/sizer.h>
+#include <wx/stattext.h>
+#include <wx/statusbr.h>
 #endif
 
 #include <wx/textfile.h>
 
 #include <math.h>
 
+#include "FreqWindow.h"
+
 #include "AColor.h"
 #include "FFT.h"
 #include "Internat.h"
 #include "PitchName.h"
+#include "Theme.h"
+#include "AllThemeResources.h"
 
+#include "FileDialog.h"
 
 enum {
    FirstID = 7000,
 
-   FreqCloseButtonID,
    FreqExportButtonID,
    FreqAlgChoiceID,
    FreqSizeChoiceID,
@@ -52,6 +85,8 @@ enum {
 
 FreqWindow *gFreqWindow = NULL;
 
+// These specify the minimum plot window width
+
 #define FREQ_WINDOW_WIDTH 480
 #define FREQ_WINDOW_HEIGHT 330
 
@@ -70,11 +105,10 @@ void InitFreqWindow(wxWindow * parent)
 
 // FreqWindow
 
-BEGIN_EVENT_TABLE(FreqWindow, wxFrame)
+BEGIN_EVENT_TABLE(FreqWindow, wxDialog)
     EVT_CLOSE(FreqWindow::OnCloseWindow)
     EVT_SIZE(FreqWindow::OnSize)
-    EVT_PAINT(FreqWindow::OnPaint)
-    EVT_BUTTON(FreqCloseButtonID, FreqWindow::OnCloseButton)
+    EVT_BUTTON(wxID_CANCEL, FreqWindow::OnCloseButton)
     EVT_BUTTON(FreqExportButtonID, FreqWindow::OnExport)
     EVT_CHOICE(FreqAlgChoiceID, FreqWindow::OnAlgChoice)
     EVT_CHOICE(FreqSizeChoiceID, FreqWindow::OnSizeChoice)
@@ -85,46 +119,22 @@ END_EVENT_TABLE()
 FreqWindow::FreqWindow(wxWindow * parent, wxWindowID id,
                            const wxString & title,
                            const wxPoint & pos):
-  wxFrame(parent, id, title, pos, wxSize(FREQ_WINDOW_WIDTH, FREQ_WINDOW_HEIGHT)),
+  wxDialog(parent, id, title, pos, wxDefaultSize,
+     wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER | wxMAXIMIZE_BOX ),
   mData(NULL), mProcessed(NULL), mBitmap(NULL)
 {
    mMouseX = 0;
    mMouseY = 0;
 
+   mFreqFont = wxFont(fontSize, wxSWISS, wxNORMAL, wxNORMAL);
    mArrowCursor = new wxCursor(wxCURSOR_ARROW);
    mCrossCursor = new wxCursor(wxCURSOR_CROSS);
 
-   mFreqPlot = new FreqPlot(this, 0,
-                            wxPoint(0, 0), wxSize(FREQ_WINDOW_WIDTH, 250));
-
-   mUpdateRect.x = 0;
-   mUpdateRect.y = 0;
-   mUpdateRect.width = FREQ_WINDOW_WIDTH;
-   mUpdateRect.height = 250;
-
-   mPlotRect.x = 10;
-   mPlotRect.y = 10;
-   mPlotRect.width = 460;
-   mPlotRect.height = 215;
-
    mLeftMargin = 40;
    mBottomMargin = 20;
 
-   mInfoRect.x = 10;
-   mInfoRect.y = 230;
-   mInfoRect.width = 460;
-   mInfoRect.height = 15;
-
-   mExportButton = new wxButton(this, FreqExportButtonID,
-                                _("Export..."),
-                                wxPoint(390, 260), wxSize(70, 20));
-
-   mCloseButton = new wxButton(this, FreqCloseButtonID,
-                               _("Close"), wxPoint(390, 290), wxSize(70, 20));
-#ifndef TARGET_CARBON
-   mCloseButton->SetDefault();
-   mCloseButton->SetFocus();
-#endif
+   mFreqPlot = new FreqPlot(this, 0,
+                            wxDefaultPosition, wxDefaultSize);
 
    wxString algChoiceStrings[5] = { _("Spectrum"),
       _("Standard Autocorrelation"),
@@ -136,25 +146,31 @@ FreqWindow::FreqWindow(wxWindow * parent, wxWindowID id,
       _("Cepstrum")
    };
 
+   wxStaticText *algLabel = new wxStaticText(this, wxID_ANY,
+                                             wxString(_("Algorithm")) + wxT(":"));
    mAlgChoice = new wxChoice(this, FreqAlgChoiceID,
-                             wxPoint(10, 260),
-                             wxSize(200, 20), 4, algChoiceStrings);
+                             wxDefaultPosition, wxDefaultSize,
+                             5, algChoiceStrings);
+   mAlgChoice->SetName(_("Algorithm"));
 
    mAlgChoice->SetSelection(0);
 
-   wxString sizeChoiceStrings[8] = { "128",
-      "256",
-      "512",
-      "1024",
-      "2048",
-      "4096",
-      "8192",
-      "16384"
+   wxString sizeChoiceStrings[8] = { wxT("128"),
+      wxT("256"),
+      wxT("512"),
+      wxT("1024"),
+      wxT("2048"),
+      wxT("4096"),
+      wxT("8192"),
+      wxT("16384")
    };
 
+   wxStaticText *sizeLabel = new wxStaticText(this, wxID_ANY,
+                                              wxString(_("Size")) + wxT(":"));
    mSizeChoice = new wxChoice(this, FreqSizeChoiceID,
-                              wxPoint(220, 260),
-                              wxSize(160, 20), 8, sizeChoiceStrings);
+                              wxDefaultPosition, wxDefaultSize,
+                              8, sizeChoiceStrings);
+   mSizeChoice->SetName(_("Size"));
 
    mSizeChoice->SetSelection(2);
 
@@ -167,9 +183,12 @@ FreqWindow::FreqWindow(wxWindow * parent, wxWindowID id,
       funcChoiceStrings[i] = WindowFuncName(i) + wxString(_(" window"));
    }
 
+   wxStaticText *funcLabel = new wxStaticText(this, wxID_ANY,
+                                              wxString(_("Function")) + wxT(":"));
    mFuncChoice = new wxChoice(this, FreqFuncChoiceID,
-                              wxPoint(10, 290),
-                              wxSize(200, 20), f, funcChoiceStrings);
+                              wxDefaultPosition, wxDefaultSize,
+                              f, funcChoiceStrings);
+   mFuncChoice->SetName(_("Function"));
 
    mFuncChoice->SetSelection(3);
    delete[]funcChoiceStrings;
@@ -178,26 +197,138 @@ FreqWindow::FreqWindow(wxWindow * parent, wxWindowID id,
       _("Log frequency")
    };
 
+   wxStaticText *axisLabel = new wxStaticText(this, wxID_ANY,
+                                              wxString(_("Axis")) + wxT(":"));
    mAxisChoice = new wxChoice(this, FreqAxisChoiceID,
-                              wxPoint(220, 290),
-                              wxSize(160, 20), 2, axisChoiceStrings);
+                              wxDefaultPosition, wxDefaultSize,
+                              2, axisChoiceStrings);
+   mAxisChoice->SetName(_("Axis"));
 
    mAxisChoice->SetSelection(0);
 
    mLogAxis = false;
 
-  #ifdef __WXMAC__
-   mBackgroundBrush.SetColour(wxColour(255, 255, 255));
-   mBackgroundPen.SetColour(wxColour(255, 255, 255));
-   SetBackgroundColour(wxColour(255, 255, 255));
-  #else
-   mBackgroundBrush.SetColour(wxColour(204, 204, 204));
-   mBackgroundPen.SetColour(wxColour(204, 204, 204));
-   SetBackgroundColour(wxColour(204, 204, 204));
-  #endif
+   mExportButton = new wxButton(this, FreqExportButtonID,
+                                _("&Export..."));
+   mExportButton->SetName(_("Export"));
 
-   // Min size, max size
-   SetSizeHints(FREQ_WINDOW_WIDTH, FREQ_WINDOW_HEIGHT, 20000, 20000);
+   mCloseButton = new wxButton(this, wxID_CANCEL,
+                               _("Close"));
+   mCloseButton->SetName(_("Close"));
+
+#ifndef TARGET_CARBON
+   mCloseButton->SetDefault();
+   mCloseButton->SetFocus();
+#endif
+
+   mInfo = new wxStatusBar(this, wxID_ANY, wxST_SIZEGRIP);
+
+   // Set minimum sizes
+
+   mFreqPlot->SetMinSize( wxSize( FREQ_WINDOW_WIDTH, FREQ_WINDOW_HEIGHT ) );
+
+   wxRect r( 0, 0, 0, 0 );
+
+   r.Union( algLabel->GetRect() );
+   r.Union( funcLabel->GetRect() );
+   algLabel->SetMinSize( r.GetSize() );
+   funcLabel->SetMinSize( r.GetSize() );
+
+   r = wxRect( 0, 0, 0, 0 );
+
+   r.Union( mAlgChoice->GetRect() );
+   r.Union( mFuncChoice->GetRect() );
+   mAlgChoice->SetMinSize( r.GetSize() );
+   mFuncChoice->SetMinSize( r.GetSize() );
+
+   r = wxRect( 0, 0, 0, 0 );
+
+   r.Union( sizeLabel->GetRect() );
+   r.Union( axisLabel->GetRect() );
+   sizeLabel->SetMinSize( r.GetSize() );
+   axisLabel->SetMinSize( r.GetSize() );
+
+   r = wxRect( 0, 0, 0, 0 );
+
+   r.Union( mSizeChoice->GetRect() );
+   r.Union( mAxisChoice->GetRect() );
+   mSizeChoice->SetMinSize( r.GetSize() );
+   mAxisChoice->SetMinSize( r.GetSize() );
+
+   r = wxRect( 0, 0, 0, 0 );
+
+   r.Union( mExportButton->GetRect() );
+   r.Union( mCloseButton->GetRect() );
+   mExportButton->SetMinSize( r.GetSize() );
+   mCloseButton->SetMinSize( r.GetSize() );
+
+   // Add everything to the sizers
+
+   wxBoxSizer *vs = new wxBoxSizer( wxVERTICAL );
+   wxBoxSizer *hs;
+
+   szr = new wxFlexGridSizer(2);
+   szr->AddGrowableCol( 1, 1 );
+   szr->AddGrowableRow( 0, 1 );
+   szr->SetFlexibleDirection( wxBOTH );
+
+   vRuler = new RulerPanel(this, wxID_ANY);
+   vRuler->ruler.SetBounds(0, 0, 100, 100); // Ruler can't handle small sizes
+   vRuler->ruler.SetOrientation(wxVERTICAL);
+   vRuler->ruler.SetRange(10.0, -90.0);
+   vRuler->ruler.SetFormat(Ruler::LinearDBFormat);
+   vRuler->ruler.SetUnits(_("dB"));
+   vRuler->ruler.SetLabelEdges(true);
+   int w, h;
+   vRuler->ruler.GetMaxSize(&w, NULL);
+   vRuler->SetSize(wxSize(w, 150));  // height needed for wxGTK
+   szr->Add( vRuler, 0, wxEXPAND|wxTOP|wxBOTTOM, 1 ); //for border around graph
+   szr->Add( mFreqPlot, 1, wxEXPAND );
+   szr->Add(1,1); //spacer on next row, under vRuler
+
+   hRuler  = new RulerPanel(this, wxID_ANY);
+   hRuler->ruler.SetBounds(0, 0, 100, 100); // Ruler can't handle small sizes
+   hRuler->ruler.SetOrientation(wxHORIZONTAL);
+   hRuler->ruler.SetLog(false);  //default for freq axis
+   hRuler->ruler.SetRange(10, 20000);  //dummy values - will get replaced
+   hRuler->ruler.SetFormat(Ruler::RealFormat);
+   hRuler->ruler.SetUnits(_("Hz"));
+   hRuler->ruler.SetFlip(true);
+   hRuler->ruler.SetLabelEdges(false);
+   hRuler->ruler.GetMaxSize(NULL, &h);
+   hRuler->SetMinSize(wxSize(-1, h));
+   szr->Add( hRuler, 0, wxEXPAND|wxLEFT|wxRIGHT, 1 ); //for border around graph
+   szr->Add(1,1); //spacer
+   mInfoText = new wxStaticText(this, wxID_ANY, wxT(""), wxDefaultPosition, wxDefaultSize, wxST_NO_AUTORESIZE);   //box for info text
+   szr->Add( mInfoText, 0, wxEXPAND|wxALL, 5);
+
+   vs->Add(szr, 1, wxEXPAND|wxALL, 5);
+
+   vs->Add( 1, 5, 0 );
+
+   hs = new wxBoxSizer( wxHORIZONTAL );
+   hs->Add( algLabel, 0, wxALIGN_CENTER | wxLEFT | wxRIGHT, 5 );
+   hs->Add( mAlgChoice, 0, wxALIGN_CENTER | wxLEFT | wxRIGHT, 5 );
+   hs->Add( sizeLabel, 0, wxALIGN_CENTER | wxLEFT | wxRIGHT, 5 );
+   hs->Add( mSizeChoice, 0, wxALIGN_CENTER | wxLEFT | wxRIGHT, 5 );
+   hs->Add( 10, 1, 0 );
+   hs->Add( mExportButton, 0, wxALIGN_CENTER | wxALIGN_RIGHT | wxLEFT | wxRIGHT, 5 );
+   vs->Add( hs, 0, wxALIGN_CENTER_HORIZONTAL | wxLEFT | wxRIGHT | wxBOTTOM, 5 );
+
+   hs = new wxBoxSizer( wxHORIZONTAL );
+   hs->Add( funcLabel, 0, wxALIGN_CENTER | wxLEFT | wxRIGHT, 5 );
+   hs->Add( mFuncChoice, 0, wxALIGN_CENTER | wxLEFT | wxRIGHT, 5 );
+   hs->Add( axisLabel, 0, wxALIGN_CENTER | wxLEFT | wxRIGHT, 5 );
+   hs->Add( mAxisChoice, 0, wxALIGN_CENTER | wxLEFT | wxRIGHT, 5 );
+   hs->Add( 10, 1, 0 );
+   hs->Add( mCloseButton, 0, wxALIGN_CENTER | wxALIGN_RIGHT | wxLEFT | wxRIGHT, 5 );
+   vs->Add( hs, 0, wxALIGN_CENTER_HORIZONTAL | wxLEFT | wxRIGHT | wxBOTTOM, 5 );
+
+   vs->Add( mInfo, 0, wxEXPAND | wxBOTTOM, 0 );
+
+   SetAutoLayout( false );
+   SetSizerAndFit( vs );
+   Layout();
 }
 
 FreqWindow::~FreqWindow()
@@ -217,65 +348,179 @@ FreqWindow::~FreqWindow()
 
 void FreqWindow::OnSize(wxSizeEvent & event)
 {
-   int width, height;
-   GetClientSize(&width, &height);
-
-   // Original size was 440 x 330
+   Layout();
 
    mUpdateRect.x = 0;
    mUpdateRect.y = 0;
-   mUpdateRect.width = width;
-   mUpdateRect.height = height - 80;
+   mUpdateRect.SetSize( mFreqPlot->GetSize() );
+   mPlotRect = mUpdateRect;
 
-   mFreqPlot->SetSize(0, 0, width, height - 80);
+   DrawPlot();
 
-   mPlotRect.x = 10;
-   mPlotRect.y = 10;
-   mPlotRect.width = width - 20;
-   mPlotRect.height = height - 115;
+   Refresh(true);
+}
 
-   mLeftMargin = 40;
-   mBottomMargin = 20;
-
-   mInfoRect.x = 10;
-   mInfoRect.y = height - 100;
-   mInfoRect.width = width - 20;
-   mInfoRect.height = 15;
-
-   mExportButton->SetSize(width - 90, height - 70, 70, 20);
-   mCloseButton->SetSize(width - 90, height - 40, 70, 20);
-
-   mAlgChoice->SetSize(10, height - 70, 160, 20);
-   mSizeChoice->SetSize(180, height - 70, 160, 20);
-   mFuncChoice->SetSize(10, height - 40, 160, 20);
-   mAxisChoice->SetSize(180, height - 40, 160, 20);
+void FreqWindow::DrawPlot()
+{
+   if (mUpdateRect.width == 0 || mUpdateRect.height == 0)
+   {
+      // Update rect not yet initialized properly
+      // (can happen during initialization phase on wx2.8)
+      return;
+   }
 
    if (mBitmap)
+   {
       delete mBitmap;
-   mBitmap = NULL;
+      mBitmap = NULL;
+   }
+
+   mBitmap = new wxBitmap(mUpdateRect.width, mUpdateRect.height);
+   
+   wxMemoryDC memDC;
+   memDC.SelectObject(*mBitmap);
+
+   memDC.SetBackground(wxBrush(wxColour(254, 254, 254)));// DONT-THEM Mask colour.
+   memDC.Clear();
+
+   wxRect r = mPlotRect;
+
+   memDC.SetPen(*wxBLACK_PEN);
+   memDC.SetBrush(*wxWHITE_BRUSH);
+   memDC.DrawRectangle(r);
+
+   if (!mProcessed) {
+      if (mData && mDataLen < mWindowSize)
+         memDC.DrawText(_("Not enough data selected."), r.x + 5, r.y + 5);
+
+      return;
+   }
+
+   float yTotal = (mYMax - mYMin);
+
+   int alg = mAlgChoice->GetSelection();
+
+   int i;
+
+   memDC.SetFont(mFreqFont);
+
+   // Set up y axis ruler
+
+   if (alg == 0) {
+      vRuler->ruler.SetUnits(_("dB"));
+      vRuler->ruler.SetFormat(Ruler::LinearDBFormat);
+   } else {
+      vRuler->ruler.SetUnits(wxT(""));
+      vRuler->ruler.SetFormat(Ruler::RealFormat);
+   }
+   int w1, w2, h;
+   vRuler->ruler.GetMaxSize(&w1, &h);
+   vRuler->ruler.SetRange(mYMax, mYMin);
+   vRuler->ruler.GetMaxSize(&w2, &h);
+   if( w1 != w2 )   // Reduces flicker
+   {
+      vRuler->SetSize(wxSize(w2,h));
+      szr->Layout();
+      hRuler->Refresh(false);
+   }
+   vRuler->Refresh(false);
+
+   // Set up x axis ruler
+
+   int width = r.width - 2;
+
+   float xMin, xMax, xPos, xRatio, xLast, xStep;
+
+   if (alg == 0) {
+      xMin = mRate / mWindowSize;
+      xMax = mRate / 2;
+      xRatio = xMax / xMin;
+      xPos = xMin;
+      xLast = xPos / 2.0;
+      if (mLogAxis)
+      {
+         xStep = pow(2.0f, (log(xRatio) / log(2.0f)) / width);
+         hRuler->ruler.SetLog(true);
+      }
+      else
+      {
+         xStep = (xMax - xMin) / width;
+         hRuler->ruler.SetLog(false);
+      }
+      hRuler->ruler.SetUnits(_("Hz"));
+   } else {
+      xMin = 0;
+      xMax = mProcessedSize / mRate;
+      xPos = xMin;
+      xLast = xPos / 2.0;
+      xStep = (xMax - xMin) / width;
+      hRuler->ruler.SetLog(false);
+      hRuler->ruler.SetUnits(_("s"));
+   }
+   hRuler->ruler.SetRange(xMin, xMax-xStep);
+   hRuler->Refresh(false);
+
+   // Draw the plot
+
+   if (alg == 0)
+      memDC.SetPen(wxPen(theTheme.Colour( clrHzPlot ), 1, wxSOLID));
+   else
+      memDC.SetPen(wxPen(theTheme.Colour( clrWavelengthPlot), 1, wxSOLID));
+
+   xPos = xMin;
+
+   for (i = 0; i < width; i++) {
+      float y;
+
+      if (mLogAxis)
+         y = GetProcessedValue(xPos, xPos * xStep);
+      else
+         y = GetProcessedValue(xPos, xPos + xStep);
+
+      float ynorm = (y - mYMin) / yTotal;
+
+      int lineheight = int (ynorm * (r.height - 1));
+
+      if (lineheight > r.height - 2)
+         lineheight = r.height - 2;
+
+      if (ynorm > 0.0)
+         memDC.DrawLine(r.x + 1 + i, r.y + r.height - 1 - lineheight,
+                        r.x + 1 + i, r.y + r.height - 1);
+
+      if (mLogAxis)
+         xPos *= xStep;
+      else
+         xPos += xStep;
+   }
+
+   // Outline the graph
+   memDC.SetPen(*wxBLACK_PEN);
+   memDC.SetBrush(*wxTRANSPARENT_BRUSH);
+   memDC.DrawRectangle(r);
+
+   memDC.SelectObject( wxNullBitmap );
+
+   mBitmap->SetMask( new wxMask( *mBitmap, wxColour(254, 254, 254) ) );
 
-#ifdef __WXMAC__
-   Refresh(true);
-#endif
 }
 
+
 void FreqWindow::PlotMouseEvent(wxMouseEvent & event)
 {
-   if (event.Moving() && event.m_x != mMouseX) {
+   if (event.Moving() && (event.m_x != mMouseX || event.m_y != mMouseY)) {
       mMouseX = event.m_x;
       mMouseY = event.m_y;
 
       wxRect r = mPlotRect;
-      r.x += mLeftMargin;
-      r.width -= mLeftMargin;
-      r.height -= mBottomMargin;
 
-      if (r.Inside(mMouseX, mMouseY))
+      if (r.Contains(mMouseX, mMouseY))
          mFreqPlot->SetCursor(*mCrossCursor);
       else
          mFreqPlot->SetCursor(*mArrowCursor);
 
       mFreqPlot->Refresh(false);
+      mInfoText->Refresh();
    }
 }
 
@@ -284,7 +529,7 @@ void FreqWindow::OnAlgChoice(wxCommandEvent & event)
    // Log-frequency axis works for spectrum plots only.
    if (mAlgChoice->GetSelection() == 0) {
       mAxisChoice->Enable(true);
-      mLogAxis = (bool)(mAxisChoice->GetSelection());
+      mLogAxis = (mAxisChoice->GetSelection())?true:false;
    }
    else {
       mAxisChoice->Disable();
@@ -305,9 +550,10 @@ void FreqWindow::OnFuncChoice(wxCommandEvent & event)
 
 void FreqWindow::OnAxisChoice(wxCommandEvent & event)
 {
-   mLogAxis = (bool)(mAxisChoice->GetSelection());
+   mLogAxis = (mAxisChoice->GetSelection())?true:false;
 
-   mFreqPlot->Refresh(false);
+   DrawPlot();
+   mFreqPlot->Refresh(true);
 }
 
 // If f(0)=y0, f(1)=y1, f(2)=y2, and f(3)=y3, this function finds
@@ -317,7 +563,7 @@ void FreqWindow::OnAxisChoice(wxCommandEvent & event)
 
 /* Declare Static functions */
 static float CubicInterpolate(float y0, float y1, float y2, float y3, float x);
-static float CubicMaximize(float y0, float y1, float y2, float y3);
+static float CubicMaximize(float y0, float y1, float y2, float y3, float * max);
 
 float CubicInterpolate(float y0, float y1, float y2, float y3, float x)
 {
@@ -334,7 +580,7 @@ float CubicInterpolate(float y0, float y1, float y2, float y3, float x)
    return (a * xxx + b * xx + c * x + d);
 }
 
-float CubicMaximize(float y0, float y1, float y2, float y3)
+float CubicMaximize(float y0, float y1, float y2, float y3, float * max)
 {
    // Find coefficients of cubic
 
@@ -370,9 +616,15 @@ float CubicMaximize(float y0, float y1, float y2, float y3)
    float ddb = db;
 
    if (dda * x1 + ddb < 0)
+   {
+      *max = a*x1*x1*x1+b*x1*x1+c*x1+d;
       return x1;
+   }
    else
+   {
+      *max = a*x2*x2*x2+b*x2*x2+c*x2+d;
       return x2;
+   }
 }
 
 float FreqWindow::GetProcessedValue(float freq0, float freq1)
@@ -421,123 +673,19 @@ float FreqWindow::GetProcessedValue(float freq0, float freq1)
    return value;
 }
 
-void FreqWindow::OnPaint(wxPaintEvent & evt)
-{
-   int width, height;
-   GetSize(&width, &height);
-
-   wxPaintDC dc(this);
-
-   dc.SetBrush(mBackgroundBrush);
-   dc.SetPen(mBackgroundPen);
-   dc.DrawRectangle(0, mUpdateRect.height,
-                    width, height - mUpdateRect.height);
-}
-
 void FreqWindow::PlotPaint(wxPaintEvent & evt)
 {
-   wxPaintDC dc(mFreqPlot);
-
-   if (!mBitmap)
-      mBitmap = new wxBitmap(mUpdateRect.width, mUpdateRect.height);
-
-   wxMemoryDC memDC;
-
-   memDC.SelectObject(*mBitmap);
-
-   memDC.SetBrush(mBackgroundBrush);
-   memDC.SetPen(mBackgroundPen);
-   memDC.DrawRectangle(0, 0, mUpdateRect.width, mUpdateRect.height);
-
-   wxRect r = mPlotRect;
-   r.x += mLeftMargin;
-   r.width -= mLeftMargin;
-   r.height -= mBottomMargin;
-
-   memDC.SetPen(*wxBLACK_PEN);
-   memDC.SetBrush(*wxWHITE_BRUSH);
-   memDC.DrawRectangle(r);
-
-   if (!mProcessed) {
-      if (mData && mDataLen < mWindowSize)
-         memDC.DrawText(_("Not enough data selected."), r.x + 5, r.y + 5);
+   wxPaintDC dc( (wxWindow *) evt.GetEventObject() );
 
+   dc.DrawBitmap( *mBitmap, 0, 0, true );
+   if( mProcessed == NULL )
       return;
-   }
-
-   float yTotal = (mYMax - mYMin);
 
    int alg = mAlgChoice->GetSelection();
 
-   int i;
+   dc.SetFont(mFreqFont);
 
-   AColor::SetLabelFont(memDC);
-
-   // Draw y axis and gridlines
-
-   if (alg == 0) {
-      for (float yi = mYMin; yi <= mYMax; yi += mYStep) {
-         i = (int) yi;
-         int y =
-             (int) (r.y + r.height - 1 -
-                    (yi - mYMin) * (r.height - 1) / yTotal);
-
-         // Light blue gridline
-         memDC.SetPen(wxPen(wxColour(204, 204, 255), 1, wxSOLID));
-         memDC.DrawLine(r.x, y, r.x + r.width, y);
-
-         // Pure blue axis line
-         memDC.SetPen(wxPen(wxColour(0, 0, 255), 1, wxSOLID));
-         memDC.DrawLine(mInfoRect.x, y, r.x, y);
-
-         if (i != (int) mYMin) {
-            wxString label = wxString::Format("%d dB", i);
-            long labelWidth, labelHeight;
-            memDC.GetTextExtent(label, &labelWidth, &labelHeight);
-            memDC.DrawText(label, r.x - labelWidth - 2, y + 1);
-         }
-      }
-   } else {
-      wxString label;
-      long labelWidth, labelHeight;
-
-      if (fabs(mYMax) < 1.0)
-         label.Printf("%.3f", mYMax);
-      else
-         label.Printf("%.1f", mYMax);
-      memDC.GetTextExtent(label, &labelWidth, &labelHeight);
-      memDC.DrawText(label, r.x - labelWidth - 2, r.y + 2);
-
-      if (fabs(mYMin) < 1.0)
-         label.Printf("%.3f", mYMin);
-      else
-         label.Printf("%.1f", mYMin);
-      memDC.GetTextExtent(label, &labelWidth, &labelHeight);
-      memDC.DrawText(label,
-                     r.x - labelWidth - 2,
-                     r.y + r.height - labelHeight - 2);
-
-      if (mYMax > 0.0 && mYMin < 0.0) {
-         int y = int ((mYMax / (mYMax - mYMin)) * r.height);
-
-         if (y > labelHeight + 4 && y < r.height - 2 * labelHeight - 4) {
-            label = "0.0";
-            memDC.GetTextExtent(label, &labelWidth, &labelHeight);
-            memDC.DrawText(label,
-                           r.x - labelWidth - 2, r.y + y - labelHeight);
-
-            // Light blue gridline
-            memDC.SetPen(wxPen(wxColour(204, 204, 255), 1, wxSOLID));
-            memDC.DrawLine(r.x, r.y + y, r.x + r.width, r.y + y);
-
-            // Pure blue axis line
-            memDC.SetPen(wxPen(wxColour(0, 0, 255), 1, wxSOLID));
-            memDC.DrawLine(mInfoRect.x, r.y + y, r.x, r.y + y);
-         }
-      }
-   }
-
-   // Draw x axis and gridlines
+   wxRect r = mPlotRect;
 
    int width = r.width - 2;
 
@@ -561,91 +709,10 @@ void FreqWindow::PlotPaint(wxPaintEvent & evt)
       xStep = (xMax - xMin) / width;
    }
 
-   int nextx = 0;
-
-   for (i = 0; i < width; i++) {
-
-      if ((mLogAxis && xPos / xLast >= 2.0) ||
-          (!mLogAxis && (i % 60) == 0)) {
-         int x = i + 1;
-
-         // Light blue gridline
-         memDC.SetPen(wxPen(wxColour(204, 204, 255), 1, wxSOLID));
-         memDC.DrawLine(r.x + x, r.y, r.x + x, r.y + r.height);
-
-         if (x >= nextx) {
-
-            // Pure blue axis line
-            memDC.SetPen(wxPen(wxColour(0, 0, 255), 1, wxSOLID));
-            memDC.DrawLine(r.x + x, r.y + r.height, r.x + x,
-                           r.y + r.height + 15);
-
-            // Label
-
-            wxString label;
-            if (alg == 0) {
-               if (xPos < 950.0)
-                  label = wxString::Format("%dHz", int (xPos + 0.5));
-               else
-                  label =
-                      wxString::Format("%dKHz",
-                                       int ((xPos / 1000.0) + 0.5));
-            } else
-               label = wxString::Format("%.4f s", xPos);
-            long labelWidth, labelHeight;
-            memDC.GetTextExtent(label, &labelWidth, &labelHeight);
-            if (x + labelWidth < width)
-               memDC.DrawText(label, r.x + x + 3, r.y + r.height + 2);
-            nextx = x + labelWidth + 4;
-         }
-
-         xLast *= 2.0;
-      }
-
-      if (mLogAxis)
-         xPos *= xStep;
-      else
-         xPos += xStep;
-   }
-
-   // Draw the plot
-
-   if (alg == 0)
-      memDC.SetPen(wxPen(wxColour(140, 60, 190), 1, wxSOLID));
-   else
-      memDC.SetPen(wxPen(wxColour(200, 50, 150), 1, wxSOLID));
-
-   xPos = xMin;
-
-   for (i = 0; i < width; i++) {
-      float y;
-
-      if (mLogAxis)
-         y = GetProcessedValue(xPos, xPos * xStep);
-      else
-         y = GetProcessedValue(xPos, xPos + xStep);
-
-      float ynorm = (y - mYMin) / yTotal;
-
-      int lineheight = int (ynorm * (r.height - 1));
-
-      if (lineheight > r.height - 2)
-         lineheight = r.height - 2;
-
-      if (ynorm > 0.0)
-         memDC.DrawLine(r.x + 1 + i, r.y + r.height - 1 - lineheight,
-                        r.x + 1 + i, r.y + r.height - 1);
-
-      if (mLogAxis)
-         xPos *= xStep;
-      else
-         xPos += xStep;
-   }
-
    // Find the peak nearest the cursor and plot it
 
    float bestpeak = float(0.0);
-   if (r.Inside(mMouseX, mMouseY)) {
+   if ( r.Contains(mMouseX, mMouseY) & (mMouseX!=0) & (mMouseX!=r.width-1) ) {
       if (mLogAxis)
          xPos = xMin * pow(xStep, mMouseX - (r.x + 1));
       else
@@ -653,6 +720,7 @@ void FreqWindow::PlotPaint(wxPaintEvent & evt)
 
       bool up = (mProcessed[1] > mProcessed[0]);
       float bestdist = 1000000;
+      float bestValue = 0.0;
       for (int bin = 2; bin < mProcessedSize; bin++) {
          bool nowUp = mProcessed[bin] > mProcessed[bin - 1];
          if (!nowUp && up) {
@@ -660,10 +728,11 @@ void FreqWindow::PlotPaint(wxPaintEvent & evt)
             int leftbin = bin - 2;
             if (leftbin < 1)
                leftbin = 1;
+            float valueAtMax = 0.0;
             float max = leftbin + CubicMaximize(mProcessed[leftbin],
                                                 mProcessed[leftbin + 1],
                                                 mProcessed[leftbin + 2],
-                                                mProcessed[leftbin + 3]);
+                                                mProcessed[leftbin + 3], &valueAtMax);
 
             float thispeak;
             if (alg == 0)
@@ -674,6 +743,7 @@ void FreqWindow::PlotPaint(wxPaintEvent & evt)
             if (fabs(thispeak - xPos) < bestdist) {
                bestpeak = thispeak;
                bestdist = fabs(thispeak - xPos);
+               bestValue = valueAtMax;
                if (thispeak > xPos)
                   break;
             }
@@ -685,36 +755,13 @@ void FreqWindow::PlotPaint(wxPaintEvent & evt)
       if (mLogAxis)
          px = int (log(bestpeak / xMin) / log(xStep));
       else
-      px = int ((bestpeak - xMin) * width / (xMax - xMin));
+         px = int ((bestpeak - xMin) * width / (xMax - xMin));
 
-      memDC.SetPen(wxPen(wxColour(160,160,160), 1, wxSOLID));
-      memDC.DrawLine(r.x + 1 + px, r.y, r.x + 1 + px, r.y + r.height);
-   }
-   // Outline the graph
+      dc.SetPen(wxPen(wxColour(160,160,160), 1, wxSOLID));
+      dc.DrawLine(r.x + 1 + px, r.y, r.x + 1 + px, r.y + r.height);
 
-   memDC.SetPen(*wxBLACK_PEN);
-   memDC.SetBrush(*wxTRANSPARENT_BRUSH);
-   memDC.DrawRectangle(r);
+       // print out info about the cursor location
 
-   // Draw the bevel around the info rect
-
-   AColor::Dark(&memDC, false);
-   memDC.DrawLine(mInfoRect.x, mInfoRect.y,
-                  mInfoRect.x + mInfoRect.width, mInfoRect.y);
-   memDC.DrawLine(mInfoRect.x, mInfoRect.y,
-                  mInfoRect.x, mInfoRect.y + mInfoRect.height);
-   AColor::Light(&memDC, false);
-   memDC.DrawLine(mInfoRect.x, mInfoRect.y + mInfoRect.height,
-                  mInfoRect.x + mInfoRect.width,
-                  mInfoRect.y + mInfoRect.height);
-   memDC.DrawLine(mInfoRect.x + mInfoRect.width, mInfoRect.y,
-                  mInfoRect.x + mInfoRect.width,
-                  mInfoRect.y + mInfoRect.height);
-
-   // If the mouse cursor is pointing inside the graph, print out info about the
-   // cursor location
-
-   if (r.Inside(mMouseX, mMouseY)) {
       float value;
 
       if (mLogAxis) {
@@ -726,33 +773,41 @@ void FreqWindow::PlotPaint(wxPaintEvent & evt)
       }
 
       wxString info;
+      wxString xpitch;
+      wxString peakpitch;
+      const wxChar *xp;
+      const wxChar *pp;
 
       if (alg == 0) {
-         wxString xpitch = PitchName_Absolute(Freq2Pitch(xPos));
-         wxString peakpitch = PitchName_Absolute(Freq2Pitch(bestpeak));
-         const char *xp = (const char *) xpitch;
-         const char *pp = (const char *) peakpitch;
-         info.Printf(_("Cursor: %d Hz (%s) = %d dB    "
-                       "Peak: %d Hz (%s)"),
-                     int (xPos + 0.5),
-                     xp, int (value + 0.5), int (bestpeak + 0.5), pp);
+         xpitch = PitchName_Absolute(Freq2Pitch(xPos));
+         peakpitch = PitchName_Absolute(Freq2Pitch(bestpeak));
+         xp = xpitch.c_str();
+         pp = peakpitch.c_str();
+
+         info.Printf(_("Cursor: %d Hz (%s) = %d dB    Peak: %d Hz (%s) = %.1f dB"),
+               int (xPos + 0.5), xp,
+               int (value + 0.5), int (bestpeak + 0.5),
+               pp, bestValue);
       } else if (xPos > 0.0 && bestpeak > 0.0) {
-         wxString xpitch = PitchName_Absolute(Freq2Pitch(1.0 / xPos));
-         wxString peakpitch = PitchName_Absolute(Freq2Pitch(1.0 / bestpeak));
-         const char *xp = (const char *) xpitch;
-         const char *pp = (const char *) peakpitch;
-         info.Printf(_("Cursor: %.4f sec (%d Hz) (%s) = %f,    "
-                       "Peak: %.4f sec (%d Hz) (%s)"),
+         xpitch = PitchName_Absolute(Freq2Pitch(1.0 / xPos));
+         peakpitch = PitchName_Absolute(Freq2Pitch(1.0 / bestpeak));
+         xp = xpitch.c_str();
+         pp = peakpitch.c_str();
+         info.Printf(_("Cursor: %.4f sec (%d Hz) (%s) = %f,    Peak: %.4f sec (%d Hz) (%s) = %.3f"),
                      xPos,
                      int (1.0 / xPos + 0.5),
-                     xp, value, bestpeak, int (1.0 / bestpeak + 0.5), pp);
+                     xp, value, bestpeak, int (1.0 / bestpeak + 0.5), pp, bestValue);
       }
-
-      memDC.DrawText(info, mInfoRect.x + 2, mInfoRect.y + 2);
+      mInfoText->SetLabel(info);
    }
+   else
+      mInfoText->SetLabel(wxT(""));
 
-   dc.Blit(0, 0, mUpdateRect.width, mUpdateRect.height,
-           &memDC, 0, 0, wxCOPY, FALSE);
+
+   // Outline the graph
+   dc.SetPen(*wxBLACK_PEN);
+   dc.SetBrush(*wxTRANSPARENT_BRUSH);
+   dc.DrawRectangle(r);
 }
 
 void FreqWindow::OnCloseWindow(wxCloseEvent & WXUNUSED(event))
@@ -775,16 +830,18 @@ void FreqWindow::Plot(int len, float *data, double rate)
    for (int i = 0; i < len; i++)
       mData[i] = data[i];
    Recalc();
+   wxSizeEvent dummy;
+   OnSize( dummy );
 }
 
 void FreqWindow::Recalc()
 {
    if (mProcessed)
-      delete mProcessed;
+      delete[] mProcessed;
    mProcessed = NULL;
 
    if (!mData) {
-      mFreqPlot->Refresh(false);
+      mFreqPlot->Refresh(true);
       return;
    }
 
@@ -793,16 +850,18 @@ void FreqWindow::Recalc()
    long windowSize = 0;
    (mSizeChoice->GetStringSelection()).ToLong(&windowSize);
 
+   int f = NumWindowFuncs();
+
    if (!(windowSize >= 32 && windowSize <= 65536 &&
-         alg >= 0 && alg <= 3 && windowFunc >= 0 && windowFunc <= 3)) {
-      mFreqPlot->Refresh(false);
+         alg >= 0 && alg <= 4 && windowFunc >= 0 && windowFunc < f)) {
+      mFreqPlot->Refresh(true);
       return;
    }
 
    mWindowSize = windowSize;
 
    if (mDataLen < mWindowSize) {
-      mFreqPlot->Refresh(false);
+      mFreqPlot->Refresh(true);
       return;
    }
 
@@ -817,14 +876,18 @@ void FreqWindow::Recalc()
    float *in2 = new float[mWindowSize];
    float *out = new float[mWindowSize];
    float *out2 = new float[mWindowSize];
+   float *win = new float[mWindowSize];
+
+   // initialize the window
+   for(int i=0; i<mWindowSize; i++)
+      win[i] = 1.0;
+   WindowFunc(windowFunc, mWindowSize, win);
 
    int start = 0;
    int windows = 0;
    while (start + mWindowSize <= mDataLen) {
       for (i = 0; i < mWindowSize; i++)
-         in[i] = mData[start + i];
-
-      WindowFunc(windowFunc, mWindowSize, in);
+         in[i] = win[i] * mData[start + i];
 
       switch (alg) {
       case 0:                  // Spectrum
@@ -868,9 +931,15 @@ void FreqWindow::Recalc()
          FFT(mWindowSize, false, in, NULL, out, out2);
 
          // Compute log power
+         float power;
          for (i = 0; i < mWindowSize; i++)
-            in[i] = log((out[i] * out[i]) + (out2[i] * out2[i]));
-
+         {
+            power = (out[i] * out[i]) + (out2[i] * out2[i]);
+            if(power <= 0.)
+               in[i] = -100000.;
+            else
+               in[i] = log((out[i] * out[i]) + (out2[i] * out2[i]));
+         }
          // Take IFFT
          FFT(mWindowSize, true, in, NULL, out, out2);
 
@@ -888,12 +957,21 @@ void FreqWindow::Recalc()
    switch (alg) {
    case 0:                     // Spectrum
       // Convert to decibels
+      mYMin = 1000000.;
+      mYMax = -1000000.;
       for (i = 0; i < half; i++)
+      {
          mProcessed[i] = 10 * log10(mProcessed[i] / mWindowSize / windows);
+         if(mProcessed[i] > mYMax)
+            mYMax = mProcessed[i];
+         else if(mProcessed[i] < mYMin)
+            mYMin = mProcessed[i];
+      }
+      if(mYMin < -90.)
+         mYMin = -90.;
+      mYMax += .5;
 
       mProcessedSize = half;
-      mYMin = -90;
-      mYMax = 10;
       mYStep = 10;
       break;
 
@@ -980,24 +1058,26 @@ void FreqWindow::Recalc()
    delete[]in2;
    delete[]out;
    delete[]out2;
+   delete[]win;
 
-   mFreqPlot->Refresh(false);
+   DrawPlot();
+   mFreqPlot->Refresh(true);
 }
 
 void FreqWindow::OnExport(wxCommandEvent & WXUNUSED(event))
 {
    wxString fName = _("spectrum.txt");
 
-   fName = wxFileSelector(_("Export Spectral Data As:"),
-                          NULL, fName, "txt", "*.txt", wxSAVE, this);
+   fName = FileSelector(_("Export Spectral Data As:"),
+                        NULL, fName, wxT("txt"), wxT("*.txt"), wxFD_SAVE | wxRESIZE_BORDER, this);
 
-   if (fName == "")
+   if (fName == wxT(""))
       return;
 
-   wxTextFile f(FILENAME(fName));
+   wxTextFile f(fName);
 #ifdef __WXMAC__
    wxFile *temp = new wxFile();
-   temp->Create(FILENAME(fName));
+   temp->Create(fName);
    delete temp;
 #else
    f.Create();
@@ -1012,13 +1092,13 @@ void FreqWindow::OnExport(wxCommandEvent & WXUNUSED(event))
       f.AddLine(_("Frequency (Hz)\tLevel (dB)"));
       for (int i = 1; i < mProcessedSize; i++)
          f.AddLine(wxString::
-                   Format("%f\t%f", i * mRate / mWindowSize,
+                   Format(wxT("%f\t%f"), i * mRate / mWindowSize,
                           mProcessed[i]));
    } else {
       f.AddLine(_("Lag (seconds)\tFrequency (Hz)\tLevel"));
       for (int i = 1; i < mProcessedSize; i++)
-         f.AddLine(wxString::Format("%f\t%f\t%f",
-                                    i / mRate, 1.0 / i, mProcessed[i]));
+         f.AddLine(wxString::Format(wxT("%f\t%f\t%f"),
+                                    i / mRate, mRate / i, mProcessed[i]));
    }
 
 #ifdef __WXMAC__
@@ -1051,3 +1131,15 @@ void FreqPlot::OnMouseEvent(wxMouseEvent & event)
 {
    freqWindow->PlotMouseEvent(event);
 }
+
+// Indentation settings for Vim and Emacs and unique identifier for Arch, a
+// version control system. Please do not modify past this point.
+//
+// Local Variables:
+// c-basic-offset: 3
+// indent-tabs-mode: nil
+// End:
+//
+// vim: et sts=3 sw=3
+// arch-tag: 8fce6648-17b9-4632-b845-3a6d25117783
+
