@@ -5,8 +5,21 @@
   Mix.cpp
 
   Dominic Mazzoni
+  Markus Meyer
+  Vaughan Johnson
+
+*******************************************************************//**
+
+\class Mixer
+\brief Functions for doing the mixdown of the tracks.
+
+*//****************************************************************//**
+
+\class MixerSpec
+\brief Class used with Mixer.
+
+*//*******************************************************************/
 
-**********************************************************************/
 
 #include "Audacity.h"
 
@@ -23,15 +36,24 @@
 #include "WaveTrack.h"
 #include "DirManager.h"
 #include "Envelope.h"
-#include "ControlToolBar.h"
+#include "Internat.h"
 #include "Prefs.h"
+#include "Project.h"
 #include "Resample.h"
+#include "float_cast.h"
 
-bool QuickMix(TrackList *tracks, TrackFactory *trackFactory,
-              double rate, sampleFormat format,
-              double startTime, double endTime,
-              WaveTrack **newLeft, WaveTrack **newRight)
+bool MixAndRender(TrackList *tracks, TrackFactory *trackFactory,
+                  double rate, sampleFormat format,
+                  double startTime, double endTime,
+                  WaveTrack **newLeft, WaveTrack **newRight)
 {
+   // This function was formerly known as "Quick Mix".  It takes one or
+   // more tracks as input; of all tracks that are selected, it mixes
+   // them together, applying any envelopes, amplitude gain, panning,
+   // and real-time effects in the process.  The resulting pair of
+   // tracks (stereo) are "rendered" and have no effects, gain, panning,
+   // or envelopes.
+
    WaveTrack **waveArray;
    Track *t;
    int numWaves = 0;
@@ -69,20 +91,19 @@ bool QuickMix(TrackList *tracks, TrackFactory *trackFactory,
       t = iter.Next();
    }
 
-   WaveTrack *mixLeft = trackFactory->NewWaveTrack(format);
-   mixLeft->SetRate(rate);
+   WaveTrack *mixLeft = trackFactory->NewWaveTrack(format, rate);
    mixLeft->SetName(_("Mix"));
    WaveTrack *mixRight = 0;
    if (mono) {
       mixLeft->SetChannel(Track::MonoChannel);
    }
    else {
-      mixRight = trackFactory->NewWaveTrack(format);
-      mixRight->SetRate(rate);
+      mixRight = trackFactory->NewWaveTrack(format, rate);
       mixRight->SetName(_("Mix"));
       mixLeft->SetChannel(Track::LeftChannel);
       mixRight->SetChannel(Track::RightChannel);
       mixLeft->SetLinked(true);
+      mixRight->SetTeamed(true);
    }
 
    int maxBlockLen = mixLeft->GetIdealBlockSize();
@@ -96,13 +117,12 @@ bool QuickMix(TrackList *tracks, TrackFactory *trackFactory,
                             startTime, endTime, mono ? 1 : 2, maxBlockLen, false,
                             rate, format);
 
-   wxProgressDialog *progress = NULL;
    wxYield();
-   wxStartTimer();
-   wxBusyCursor busy;
+   ProgressDialog *progress = new ProgressDialog(_NoAcc("&Mix and Render"),
+                                                 _("Mixing and rendering tracks"));
    
-   bool cancelling = false;
-   while(!cancelling) {
+   bool bCancel = false;
+   while(!bCancel) {
       sampleCount blockLen = mixer->Process(maxBlockLen);
 
       if (blockLen == 0)
@@ -120,25 +140,24 @@ bool QuickMix(TrackList *tracks, TrackFactory *trackFactory,
          mixRight->Append(buffer, format, blockLen);
       }
 
-      if (!progress && wxGetElapsedTime(false) > 500) {
-         progress =
-            new wxProgressDialog(_("Quick Mix"), _("Mixing tracks"), 1000);
-      }
-      if (progress) {
-         int progressvalue = int (1000 * (mixer->MixGetCurrentTime() / totalTime));
-         cancelling = !progress->Update(progressvalue);
-      }
-   }
-
-   mixLeft->Flush();
-   *newLeft = mixLeft;
-   if (!mono) {
-      mixRight->Flush();
-      *newRight = mixRight;
+      bCancel = !progress->Update(mixer->MixGetCurrentTime(), totalTime);
    }
 
    delete progress;
 
+   mixLeft->Flush();
+   if (!mono) 
+      mixRight->Flush();
+   if (bCancel)
+   {
+      delete mixLeft;
+      if (!mono) 
+         delete mixRight;
+   } else {
+      *newLeft = mixLeft;
+      if (!mono) 
+         *newRight = mixRight;
+
 #if 0
    int elapsedMS = wxGetElapsedTime();
    double elapsedTime = elapsedMS * 0.001;
@@ -151,11 +170,12 @@ bool QuickMix(TrackList *tracks, TrackFactory *trackFactory,
    printf("Elapsed time: %f sec\n", elapsedTime);
    printf("Max number of tracks to mix in real time: %f\n", maxTracks);
 #endif
+   }
 
    delete[] waveArray;
    delete mixer;
 
-   return true;
+   return !bCancel;
 }
 
 Mixer::Mixer(int numInputTracks, WaveTrack **inputTracks,
@@ -163,13 +183,13 @@ Mixer::Mixer(int numInputTracks, WaveTrack **inputTracks,
              double startTime, double stopTime,
              int numOutChannels, int outBufferSize, bool outInterleaved,
              double outRate, sampleFormat outFormat,
-             bool highQuality)
+             bool highQuality, MixerSpec *mixerSpec)
 {
    int i;
 
    mNumInputTracks = numInputTracks;
    mInputTrack = new WaveTrack*[mNumInputTracks];
-   mSamplePos = new longSampleCount[mNumInputTracks];
+   mSamplePos = new sampleCount[mNumInputTracks];
    for(i=0; i<mNumInputTracks; i++) {
       mInputTrack[i] = inputTracks[i];
       mSamplePos[i] = inputTracks[i]->TimeToLongSamples(startTime);
@@ -185,6 +205,11 @@ Mixer::Mixer(int numInputTracks, WaveTrack **inputTracks,
    mFormat = outFormat;
    mApplyTrackGains = true;
    mGains = new float[mNumChannels];
+   if( mixerSpec && mixerSpec->GetNumChannels() == mNumChannels &&
+         mixerSpec->GetNumTracks() == mNumInputTracks )
+      mMixerSpec = mixerSpec;
+   else
+      mMixerSpec = NULL;
 
    if (mInterleaved) {
       mNumBuffers = 1;
@@ -196,9 +221,11 @@ Mixer::Mixer(int numInputTracks, WaveTrack **inputTracks,
    }
    
    mBuffer = new samplePtr[mNumBuffers];
-   for (int c = 0; c < mNumBuffers; c++)
+   mTemp = new samplePtr[mNumBuffers];
+   for (int c = 0; c < mNumBuffers; c++) {
       mBuffer[c] = NewSamples(mInterleavedBufferSize, mFormat);
-   mTemp = NewSamples(mInterleavedBufferSize, mFormat);
+      mTemp[c] = NewSamples(mInterleavedBufferSize, floatSample);
+   }
    mFloatBuffer = new float[mInterleavedBufferSize];
 
    mQueueMaxLen = 65536;
@@ -231,9 +258,10 @@ Mixer::~Mixer()
 {
    int i;
 
-   DeleteSamples(mTemp);
-   for (i = 0; i < mNumBuffers; i++)
+   for (i = 0; i < mNumBuffers; i++) {
       DeleteSamples(mBuffer[i]);
+      DeleteSamples(mTemp[i]);
+   }
    delete[] mBuffer;
    delete[] mInputTrack;
    delete[] mEnvValues;
@@ -258,12 +286,13 @@ void Mixer::ApplyTrackGains(bool apply)
 
 void Mixer::Clear()
 {
-   for (int c = 0; c < mNumBuffers; c++)
-      memset(mBuffer[c], 0, mInterleavedBufferSize * SAMPLE_SIZE(mFormat));
+   for (int c = 0; c < mNumBuffers; c++) {
+      memset(mTemp[c], 0, mInterleavedBufferSize * SAMPLE_SIZE(floatSample));
+   }
 }
 
 void MixBuffers(int numChannels, int *channelFlags, float *gains,
-                sampleFormat format, samplePtr src, samplePtr *dests,
+                samplePtr src, samplePtr *dests,
                 int len, bool interleaved)
 {
    for (int c = 0; c < numChannels; c++) {
@@ -274,7 +303,7 @@ void MixBuffers(int numChannels, int *channelFlags, float *gains,
       int skip;
       
       if (interleaved) {
-         destPtr = dests[0] + c*SAMPLE_SIZE(format);
+         destPtr = dests[0] + c*SAMPLE_SIZE(floatSample);
          skip = numChannels;
       } else {
          destPtr = dests[c];
@@ -282,92 +311,71 @@ void MixBuffers(int numChannels, int *channelFlags, float *gains,
       }
 
       float gain = gains[c];
-
-      switch(format) {
-      case int16Sample: {
-         short *dest = (short *)destPtr;
-         short *temp = (short *)src;
-         for (int j = 0; j < len; j++) {
-            float f = temp[j] * gain + *dest;
-            if (f > 32767)
-               f = 32767;
-            if (f < -32768)
-               f = -32768;
-            *dest = (short)f;
-            dest += skip;
-         }
-      } break;
-      case int24Sample: {
-         int *dest = (int *)destPtr;
-         int *temp = (int *)src;
-         for (int j = 0; j < len; j++) {
-            float f = temp[j] * gain + *dest;
-            if (f > 8388607)
-               f = 8388607;
-            if (f < -8388608)
-               f = -8388608;
-            *dest = (int)f;
-            dest += skip;
-         }
-      } break;
-      case floatSample: {
-         float *dest = (float *)destPtr;
-         float *temp = (float *)src;
-         for (int j = 0; j < len; j++) {
-            float f = *dest + temp[j] * gain;
-            // MM: XXX Should probably go outside the loop
-            if (f > 1.0f)
-               *dest = 1.0f;
-            else if (f < -1.0f)
-               *dest = -1.0f;
-            else
-               *dest = f;
-            dest += skip;
-         }
-      } break;
-      } // switch
+      float *dest = (float *)destPtr;
+      float *temp = (float *)src;
+      for (int j = 0; j < len; j++) {
+         *dest += temp[j] * gain;   // the actual mixing process
+         dest += skip;
+      }
    }
 }
    
 sampleCount Mixer::MixVariableRates(int *channelFlags, WaveTrack *track,
-                                    longSampleCount *pos, float *queue,
+                                    sampleCount *pos, float *queue,
                                     int *queueStart, int *queueLen,
                                     Resample *SRC)
 {
-   double initialWarp = mRate / track->GetRate();
-   double t = *pos / track->GetRate();
+   int trackRate = (int)(track->GetRate());
+   double initialWarp = mRate / trackRate;
+   double t = *pos / trackRate;
    int sampleSize = SAMPLE_SIZE(floatSample);
    int i, c;
 
    sampleCount out = 0;
 
+   // Find the last sample
+   sampleCount last = -1;
+   WaveClipList::Node* it = track->GetClipIterator();
+   while (it) {
+      sampleCount end = it->GetData()->GetEndSample();
+      if (end > last) {
+         last = end;
+      }
+      it = it->GetNext();
+   }
+
+   sampleCount max = (sampleCount) (trackRate * mT1);
+   /* time is floating point. Sample rate is integer. The number of samples
+    * has to be integer, but the multiplication gives a float result, which we
+    * round to get an integer result. TODO: is this always right or can it be
+    * off by one sometimes? Can we not get this information directly from the
+    * clip (which must know) rather than convert the time? */
+   if (last > max)
+      last = max;
+
    while(out < mMaxOut) {
       if (*queueLen < mProcessLen) {
-         memmove(queue, &queue[*queueStart],
-                 (*queueLen)*sampleSize);
+         memmove(queue, &queue[*queueStart], (*queueLen)*sampleSize);
          *queueStart = 0;
 
          int getLen = mQueueMaxLen - *queueLen;
 
-         #if 0
+         // Constrain
+         if (*pos + getLen > last) {
+            getLen = last - *pos;
+         }
 
-         // TODO: fix this code so that extra silence isn't added
-         // to the end of a track
-
-         double trackTime = (*pos + getLen) / track->GetRate();
-         if (trackTime > track->GetEndTime()) {
-            getLen = (int)(0.5 + track->GetRate() *
-                           (track->GetEndTime() -
-                            ((*pos) / track->GetRate())));
+         // Nothing to do if past end of track
+         if (getLen <= 0) {
+            break;
          }
-         #endif
 
          track->Get((samplePtr)&queue[*queueLen], floatSample,
                     *pos, getLen);
 
-         Envelope *e = track->GetEnvelope();
-         e->GetValues(mEnvValues, getLen, (*pos) / track->GetRate(),
-                      1.0 / track->GetRate());
+         track->GetEnvelopeValues(mEnvValues, getLen, (*pos) / trackRate,
+                                  1.0 / trackRate);
+
          for(i=0; i<getLen; i++)
             queue[(*queueLen)+i] *= mEnvValues[i];
          *queueLen += getLen;
@@ -403,30 +411,26 @@ sampleCount Mixer::MixVariableRates(int *channelFlags, WaveTrack *track,
       *queueStart += input_used;
       *queueLen -= input_used;
       out += outgen;
-      t += (input_used / track->GetRate());
+      t += (input_used / trackRate);
 
       if (last)
          break;
    }
 
-   CopySamples((samplePtr)mFloatBuffer, floatSample,
-               mTemp, mFormat,
-               mMaxOut);
-
    for(c=0; c<mNumChannels; c++)
       if (mApplyTrackGains)
          mGains[c] = track->GetChannelGain(c);
       else
          mGains[c] = 1.0;
 
-   MixBuffers(mNumChannels, channelFlags, mGains, mFormat,
-              mTemp, mBuffer, mMaxOut, mInterleaved);
+   MixBuffers(mNumChannels, channelFlags, mGains,
+              (samplePtr)mFloatBuffer, mTemp, out, mInterleaved);
 
-   return mMaxOut;
+   return out;
 }
 
 sampleCount Mixer::MixSameRate(int *channelFlags, WaveTrack *track,
-                               longSampleCount *pos)
+                               sampleCount *pos)
 {
    int slen = mMaxOut;
    int c;
@@ -442,40 +446,29 @@ sampleCount Mixer::MixSameRate(int *channelFlags, WaveTrack *track,
       slen = mMaxOut;
 
    track->Get((samplePtr)mFloatBuffer, floatSample, *pos, slen);
-   Envelope *e = track->GetEnvelope();
-   e->GetValues(mEnvValues, slen, t, 1.0 / mRate);
+   track->GetEnvelopeValues(mEnvValues, slen, t, 1.0 / mRate);
    for(int i=0; i<slen; i++)
       mFloatBuffer[i] *= mEnvValues[i]; // Track gain control will go here?
    
-   if (mFormat == track->GetSampleFormat())
-   {
-      CopySamplesNoDither((samplePtr)mFloatBuffer, floatSample,
-                          mTemp, mFormat, slen);
-   } else {
-      CopySamples((samplePtr)mFloatBuffer, floatSample,
-                          mTemp, mFormat, slen);
-   }
-   
-
    for(c=0; c<mNumChannels; c++)
       if (mApplyTrackGains)
          mGains[c] = track->GetChannelGain(c);
       else
          mGains[c] = 1.0;
 
-   MixBuffers(mNumChannels, channelFlags, mGains, mFormat,
-              mTemp, mBuffer, slen, mInterleaved);
+   MixBuffers(mNumChannels, channelFlags, mGains,
+              (samplePtr)mFloatBuffer, mTemp, slen, mInterleaved);
 
    *pos += slen;
 
    return slen;
 }
 
-sampleCount Mixer::Process(int maxToProcess)
+sampleCount Mixer::Process(sampleCount maxToProcess)
 {
    if (mT >= mT1)
       return 0;
-   
+
    int i, j;
    sampleCount out;
    sampleCount maxOut = 0;
@@ -489,21 +482,28 @@ sampleCount Mixer::Process(int maxToProcess)
       for(j=0; j<mNumChannels; j++)
          channelFlags[j] = 0;
 
-      switch(track->GetChannel()) {
-      case Track::MonoChannel:
-      default:
-         for(j=0; j<mNumChannels; j++)
-            channelFlags[j] = 1;
-         break;
-      case Track::LeftChannel:
-         channelFlags[0] = 1;
-         break;
-      case Track::RightChannel:
-         if (mNumChannels >= 2)
-            channelFlags[1] = 1;
-         else
+      if( mMixerSpec ) {
+         //ignore left and right when downmixing is not required
+         for( j = 0; j < mNumChannels; j++ )
+            channelFlags[ j ] = mMixerSpec->mMap[ i ][ j ] ? 1 : 0;
+      }
+      else {
+         switch(track->GetChannel()) {
+         case Track::MonoChannel:
+         default:
+            for(j=0; j<mNumChannels; j++)
+               channelFlags[j] = 1;
+            break;
+         case Track::LeftChannel:
             channelFlags[0] = 1;
-         break;
+            break;
+         case Track::RightChannel:
+            if (mNumChannels >= 2)
+               channelFlags[1] = 1;
+            else
+               channelFlags[0] = 1;
+            break;
+         }
       }
 
       if (mTimeTrack ||
@@ -517,6 +517,9 @@ sampleCount Mixer::Process(int maxToProcess)
       if (out > maxOut)
          maxOut = out;
    }
+   out = mInterleaved ? maxOut * mNumChannels : maxOut;
+   for(int c=0; c<mNumBuffers; c++)
+      CopySamples(mTemp[c], floatSample, mBuffer[c], mFormat, out);
 
    mT += (maxOut / mRate);
 
@@ -554,3 +557,118 @@ void Mixer::Restart()
       mQueueLen[i] = 0;
    }
 }
+
+void Mixer::Reposition(double t)
+{
+   int i;
+
+   mT = t;
+   if( mT < mT0 )
+      mT = mT0;
+   if( mT > mT1 )
+      mT = mT1;
+
+   for(i=0; i<mNumInputTracks; i++) {
+      mSamplePos[i] = mInputTrack[i]->TimeToLongSamples(mT);
+      mQueueStart[i] = 0;
+      mQueueLen[i] = 0;
+   }
+}
+
+MixerSpec::MixerSpec( int numTracks, int maxNumChannels )
+{
+   mNumTracks = mNumChannels = numTracks;
+   mMaxNumChannels = maxNumChannels;
+   
+   if( mNumChannels > mMaxNumChannels )
+         mNumChannels = mMaxNumChannels;
+   
+   Alloc();
+
+   for( int i = 0; i < mNumTracks; i++ )
+      for( int j = 0; j < mNumChannels; j++ )
+         mMap[ i ][ j ] = ( i == j );
+}
+
+MixerSpec::MixerSpec( const MixerSpec &mixerSpec )
+{
+   mNumTracks = mixerSpec.mNumTracks;
+   mMaxNumChannels = mixerSpec.mMaxNumChannels;
+   mNumChannels = mixerSpec.mNumChannels;
+
+   Alloc();
+
+   for( int i = 0; i < mNumTracks; i++ )
+      for( int j = 0; j < mNumChannels; j++ )
+         mMap[ i ][ j ] = mixerSpec.mMap[ i ][ j ];
+}
+
+void MixerSpec::Alloc()
+{
+   mMap = new bool*[ mNumTracks ];
+   for( int i = 0; i < mNumTracks; i++ )
+      mMap[ i ] = new bool[ mMaxNumChannels ];
+}
+
+MixerSpec::~MixerSpec()
+{
+   Free();
+}
+
+void MixerSpec::Free()
+{
+   for( int i = 0; i < mNumTracks; i++ )
+      delete[] mMap[ i ];
+
+   delete[] mMap;
+}
+
+bool MixerSpec::SetNumChannels( int newNumChannels )
+{
+   if( mNumChannels == newNumChannels )
+      return true;
+
+   if( newNumChannels > mMaxNumChannels )
+      return false;
+   
+   for( int i = 0; i < mNumTracks; i++ )
+   {
+      for( int j = newNumChannels; j < mNumChannels; j++ )
+         mMap[ i ][ j ] = false;
+
+      for( int j = mNumChannels; j < newNumChannels; j++ )
+         mMap[ i ][ j ] = false;
+   }
+
+   mNumChannels = newNumChannels;
+   return true;
+}
+
+MixerSpec& MixerSpec::operator=( const MixerSpec &mixerSpec )
+{
+   Free();
+   
+   mNumTracks = mixerSpec.mNumTracks;
+   mNumChannels = mixerSpec.mNumChannels;
+   mMaxNumChannels = mixerSpec.mMaxNumChannels;
+
+   Alloc();
+
+   for( int i = 0; i < mNumTracks; i++ )
+      for( int j = 0; j < mNumChannels; j++ )
+         mMap[ i ][ j ] = mixerSpec.mMap[ i ][ j ];
+
+   return *this;
+}
+
+// Indentation settings for Vim and Emacs and unique identifier for Arch, a
+// version control system. Please do not modify past this point.
+//
+// Local Variables:
+// c-basic-offset: 3
+// indent-tabs-mode: nil
+// End:
+//
+// vim: et sts=3 sw=3
+// arch-tag: d4e10e74-cdf9-46ac-b309-91b115d2a78f
+
