@@ -1,14 +1,14 @@
 /**********************************************************************
 
-  Audacity: A Digital Audio Editor
+Audacity: A Digital Audio Editor
 
-  SoundTouchEffect.cpp
+SoundTouchEffect.cpp
 
-  Dominic Mazzoni, Vaughan Johnson
+Dominic Mazzoni, Vaughan Johnson
 
-  This abstract class contains all of the common code for an
-  effect that uses SoundTouch to do its processing (ChangeTempo
-  and ChangePitch).
+This abstract class contains all of the common code for an
+effect that uses SoundTouch to do its processing (ChangeTempo
+                                                  and ChangePitch).
 
 **********************************************************************/
 
@@ -20,48 +20,79 @@
 
 #include "SoundTouchEffect.h"
 #include "../WaveTrack.h"
+#include "../Project.h"
 
 
 bool EffectSoundTouch::Process()
 {
    // Assumes that mSoundTouch has already been initialized
    // by the subclass for subclass-specific parameters.
-
+   
    //Iterate over each track
-   TrackListIterator iter(mWaveTracks);
+   this->CopyInputWaveTracks(); // Set up mOutputWaveTracks.
+   bool bGoodResult = true;
+
+   TrackListIterator iter(mOutputWaveTracks);
    WaveTrack* leftTrack = (WaveTrack*)(iter.First());
-   WaveTrack* rightTrack = NULL;
    mCurTrackNum = 0;
 	m_maxNewLength = 0.0;
-   while (leftTrack) {
+	
+   double len = leftTrack->GetEndTime() - leftTrack->GetStartTime();
+   
+   while (leftTrack != NULL) {
       //Get start and end times from track
-      double trackStart = leftTrack->GetStartTime();
-      double trackEnd = leftTrack->GetEndTime();
-
+      mCurT0 = leftTrack->GetStartTime();
+      mCurT1 = leftTrack->GetEndTime();
+      
       //Set the current bounds to whichever left marker is
-      //greater and whichever right marker is less:
-      mCurT0 = mT0 < trackStart? trackStart: mT0;
-      mCurT1 = mT1 > trackEnd? trackEnd: mT1;
-
+      //greater and whichever right marker is less
+      mCurT0 = wxMax(mT0, mCurT0);
+      mCurT1 = wxMin(mT1, mCurT1);
+      
       // Process only if the right marker is to the right of the left marker
       if (mCurT1 > mCurT0) {
-
-         //Transform the marker timepoints to samples
-         longSampleCount start = leftTrack->TimeToLongSamples(mCurT0);
-         longSampleCount end = leftTrack->TimeToLongSamples(mCurT1);
+         sampleCount start, end;
          
-         rightTrack = NULL; 
          if (leftTrack->GetLinked()) {
-            rightTrack = (WaveTrack*)(iter.Next());
+            double t;
+            WaveTrack* rightTrack = (WaveTrack*)(iter.Next());
+
+            //Adjust bounds by the right tracks markers
+            t = rightTrack->GetStartTime();
+            t = wxMax(mT0, t);
+            mCurT0 = wxMin(mCurT0, t);
+            t = rightTrack->GetEndTime();
+            t = wxMin(mT1, t);
+            mCurT1 = wxMax(mCurT1, t);
+
+            //Transform the marker timepoints to samples
+            start = leftTrack->TimeToLongSamples(mCurT0);
+            end = leftTrack->TimeToLongSamples(mCurT1);
+
+            //Inform soundtouch there's 2 channels
             mSoundTouch->setChannels(2);
+
+            //ProcessStereo() (implemented below) processes a stereo track
             if (!ProcessStereo(leftTrack, rightTrack, start, end))
-               return false;
+            {
+               bGoodResult = false;
+               break;
+            }
             mCurTrackNum++; // Increment for rightTrack, too.
          } else {
+            //Transform the marker timepoints to samples
+            start = leftTrack->TimeToLongSamples(mCurT0);
+            end = leftTrack->TimeToLongSamples(mCurT1);
+
+            //Inform soundtouch there's a single channel
             mSoundTouch->setChannels(1);
+
             //ProcessOne() (implemented below) processes a single track
             if (!ProcessOne(leftTrack, start, end))
-               return false;
+            {
+               bGoodResult = false;
+               break;
+            }
          }
       }
       
@@ -70,51 +101,67 @@ bool EffectSoundTouch::Process()
       mCurTrackNum++;
    }
 
+   this->ReplaceProcessedWaveTracks(bGoodResult); 
+
+#ifdef EXPERIMENTAL_FULL_LINKING
+   AudacityProject *p = (AudacityProject*)mParent;
+   if( p && p->IsSticky() ){
+      leftTrack = (WaveTrack*)(iter.First());
+      double newLen = leftTrack->GetEndTime() - leftTrack->GetStartTime();
+      double timeAdded = newLen-len;
+      double sel = mCurT1-mCurT0;
+      double percent = (sel/(timeAdded+sel))*100 - 100;
+      if ( !(HandleGroupChangeSpeed(percent, mCurT0, mCurT1)) ) bGoodResult = false;
+   }
+#endif
+
    delete mSoundTouch;
    mSoundTouch = NULL;
+   
+//   mT0 = mCurT0;
+//   mT1 = mCurT0 + m_maxNewLength; // Update selection.
 
-	mT1 = mT0 + m_maxNewLength; // Update selection.
-   return true;
+   return bGoodResult;
 }
 
 //ProcessOne() takes a track, transforms it to bunch of buffer-blocks,
 //and executes ProcessSoundTouch on these blocks
 bool EffectSoundTouch::ProcessOne(WaveTrack *track,
-                                  longSampleCount start, longSampleCount end)
+                                  sampleCount start, sampleCount end)
 {
    WaveTrack *outputTrack;
-   longSampleCount s;
-
+   sampleCount s;
+   
    mSoundTouch->setSampleRate((unsigned int)(track->GetRate()+0.5));
    
-   outputTrack = mFactory->NewWaveTrack(track->GetSampleFormat());
-
+   outputTrack = mFactory->NewWaveTrack(track->GetSampleFormat(), track->GetRate());
+   
    //Get the length of the buffer (as double). len is
    //used simple to calculate a progress meter, so it is easier
    //to make it a double now than it is to do it later 
    double len = (double)(end - start);
-
+   
    //Initiate a processing buffer.  This buffer will (most likely)
    //be shorter than the length of the track being processed.
    float *buffer = new float[track->GetMaxBlockSize()];
-
+   
    //Go through the track one buffer at a time. s counts which
    //sample the current buffer starts at.
    s = start;
    while (s < end) {
       //Get a block of samples (smaller than the size of the buffer)
       sampleCount block = track->GetBestBlockSize(s);
-
+      
       //Adjust the block size if it is the final block in the track
       if (s + block > end)
          block = end - s;
-
+      
       //Get the samples from the track and put them in the buffer
       track->Get((samplePtr) buffer, floatSample, s, block);
-
+      
       //Add samples to SoundTouch
       mSoundTouch->putSamples(buffer, block);
-
+      
       //Get back samples from SoundTouch
       unsigned int outputCount = mSoundTouch->numSamples();
       if (outputCount > 0) {
@@ -123,18 +170,18 @@ bool EffectSoundTouch::ProcessOne(WaveTrack *track,
          outputTrack->Append((samplePtr)buffer2, floatSample, outputCount);
          delete[] buffer2;
       }
-
+      
       //Increment s one blockfull of samples
       s += block;
-
+      
       //Update the Progress meter
       if (TrackProgress(mCurTrackNum, (s - start) / len))
          return false;
    }
-
+   
    // Tell SoundTouch to finish processing any remaining samples
    mSoundTouch->flush();
-
+   
    unsigned int outputCount = mSoundTouch->numSamples();
    if (outputCount > 0) {
       float *buffer2 = new float[outputCount];
@@ -142,43 +189,44 @@ bool EffectSoundTouch::ProcessOne(WaveTrack *track,
       outputTrack->Append((samplePtr)buffer2, floatSample, outputCount);
       delete[] buffer2;
    }
-
+   
    // Flush the output WaveTrack (since it's buffered, too)
    outputTrack->Flush();
-
+   
    // Clean up the buffer
    delete[]buffer;
-
+   
    // Take the output track and insert it in place of the original
    // sample data
-
-   track->Clear(mT0, mT1);
-   track->Paste(mT0, outputTrack);
-
-	double newLength = outputTrack->GetEndTime(); 
-	if (newLength > m_maxNewLength) 
-		m_maxNewLength = newLength; 
-
+   
+   track->HandleClear(mCurT0, mCurT1, false, false);
+   track->HandlePaste(mCurT0, outputTrack);
+   
+   double newLength = outputTrack->GetEndTime(); 
+   m_maxNewLength = wxMax(m_maxNewLength, newLength);
+   
    // Delete the outputTrack now that its data is inserted in place
    delete outputTrack;
-
+   
    //Return true because the effect processing succeeded.
    return true;
 }
 
 bool EffectSoundTouch::ProcessStereo(WaveTrack* leftTrack, WaveTrack* rightTrack, 
-                                       longSampleCount start, longSampleCount end)
+                                     sampleCount start, sampleCount end)
 {
    mSoundTouch->setSampleRate((unsigned int)(leftTrack->GetRate()+0.5));
    
-   WaveTrack* outputLeftTrack = mFactory->NewWaveTrack(leftTrack->GetSampleFormat());
-   WaveTrack* outputRightTrack = mFactory->NewWaveTrack(rightTrack->GetSampleFormat());
-
+   WaveTrack* outputLeftTrack = mFactory->NewWaveTrack(leftTrack->GetSampleFormat(),
+                                                       leftTrack->GetRate());
+   WaveTrack* outputRightTrack = mFactory->NewWaveTrack(rightTrack->GetSampleFormat(),
+                                                        rightTrack->GetRate());
+   
    //Get the length of the buffer (as double). len is
    //used simple to calculate a progress meter, so it is easier
    //to make it a double now than it is to do it later 
    double len = (double)(end - start);
-
+   
    //Initiate a processing buffer.  This buffer will (most likely)
    //be shorter than the length of the track being processed.
    // Make soundTouchBuffer twice as big as MaxBlockSize for each channel, 
@@ -188,76 +236,89 @@ bool EffectSoundTouch::ProcessStereo(WaveTrack* leftTrack, WaveTrack* rightTrack
    float* leftBuffer = new float[maxBlockSize];
    float* rightBuffer = new float[maxBlockSize];
    float* soundTouchBuffer = new float[maxBlockSize * 2];
-
+   
    // Go through the track one stereo buffer at a time. 
    // sourceSampleCount counts the sample at which the current buffer starts, 
    // per channel.
-   longSampleCount sourceSampleCount = start;
+   sampleCount sourceSampleCount = start;
    while (sourceSampleCount < end) {
       //Get a block of samples (smaller than the size of the buffer)
       sampleCount blockSize = leftTrack->GetBestBlockSize(sourceSampleCount);
-
+      
       //Adjust the block size if it is the final block in the track
       if (sourceSampleCount + blockSize > end)
          blockSize = end - sourceSampleCount;
-
+      
       // Get the samples from the tracks and put them in the buffers.
       leftTrack->Get((samplePtr)(leftBuffer), floatSample, sourceSampleCount, blockSize);
       rightTrack->Get((samplePtr)(rightBuffer), floatSample, sourceSampleCount, blockSize);
-
+      
       // Interleave into soundTouchBuffer.
       for (int index = 0; index < blockSize; index++) {
          soundTouchBuffer[index*2]       = leftBuffer[index];
          soundTouchBuffer[(index*2)+1]   = rightBuffer[index];
       }
-
+      
       //Add samples to SoundTouch
       mSoundTouch->putSamples(soundTouchBuffer, blockSize);
-
+      
       //Get back samples from SoundTouch
       unsigned int outputCount = mSoundTouch->numSamples();
       if (outputCount > 0) 
          this->ProcessStereoResults(outputCount, outputLeftTrack, outputRightTrack);
-
+      
       //Increment sourceSampleCount one blockfull of samples
       sourceSampleCount += blockSize;
-
+      
       //Update the Progress meter
-      if (TrackProgress(mCurTrackNum, (sourceSampleCount - start) / len))
+      // mCurTrackNum is left track. Include right track. 
+      int nWhichTrack = mCurTrackNum;
+      double frac = (sourceSampleCount - start) / len;
+      if (frac < 0.5)
+         frac *= 2.0; // Show twice as far for each track, because we're doing 2 at once. 
+      else 
+      {
+         nWhichTrack++;
+         frac -= 0.5;
+         frac *= 2.0; // Show twice as far for each track, because we're doing 2 at once. 
+      }
+      if (TrackProgress(nWhichTrack, frac))
          return false;
    }
-
+   
    // Tell SoundTouch to finish processing any remaining samples
    mSoundTouch->flush();
-
+   
    unsigned int outputCount = mSoundTouch->numSamples();
    if (outputCount > 0) 
       this->ProcessStereoResults(outputCount, outputLeftTrack, outputRightTrack);
-
+   
    // Flush the output WaveTracks (since they're buffered, too)
    outputLeftTrack->Flush();
    outputRightTrack->Flush();
-
+   
    // Clean up the buffers.
    delete [] leftBuffer;
    delete [] rightBuffer;
    delete [] soundTouchBuffer;
-
+   
    // Take the output tracks and insert in place of the original
    // sample data.
-   leftTrack->Clear(mT0, mT1);
-   leftTrack->Paste(mT0, outputLeftTrack);
-   rightTrack->Clear(mT0, mT1);
-   rightTrack->Paste(mT0, outputRightTrack);
-
-	double newLength = outputLeftTrack->GetEndTime(); 
-	if (newLength > m_maxNewLength) 
-		m_maxNewLength = newLength; 
+   leftTrack->HandleClear(mCurT0, mCurT1, false, false);
+   leftTrack->HandlePaste(mCurT0, outputLeftTrack);
+   rightTrack->HandleClear(mCurT0, mCurT1, false, false);
+   rightTrack->HandlePaste(mCurT0, outputRightTrack);
 
+   // Track the longest result length
+   double newLength = outputLeftTrack->GetEndTime();
+   m_maxNewLength = wxMax(m_maxNewLength, newLength);
+   newLength = outputRightTrack->GetEndTime();
+   m_maxNewLength = wxMax(m_maxNewLength, newLength);
+   
    // Delete the outputTracks now that their data are inserted in place.
    delete outputLeftTrack;
    delete outputRightTrack;
-
+   
    //Return true because the effect processing succeeded.
    return true;
 }
@@ -268,23 +329,23 @@ bool EffectSoundTouch::ProcessStereoResults(const unsigned int outputCount,
 {
    float* outputSoundTouchBuffer = new float[outputCount*2];
    mSoundTouch->receiveSamples(outputSoundTouchBuffer, outputCount);
-
+   
    // Dis-interleave outputSoundTouchBuffer into separate track buffers.
    float* outputLeftBuffer = new float[outputCount];
    float* outputRightBuffer = new float[outputCount];
-   for (int index = 0; index < outputCount; index++)
+   for (unsigned int index = 0; index < outputCount; index++)
    {
       outputLeftBuffer[index] = outputSoundTouchBuffer[index*2];
       outputRightBuffer[index] = outputSoundTouchBuffer[(index*2)+1];
    }
-
+   
    outputLeftTrack->Append((samplePtr)outputLeftBuffer, floatSample, outputCount);
    outputRightTrack->Append((samplePtr)outputRightBuffer, floatSample, outputCount);
-
+   
    delete[] outputSoundTouchBuffer;
    delete[] outputLeftBuffer;
    delete[] outputRightBuffer;
-
+   
    return true;
 }
 
