@@ -62,7 +62,7 @@ private:
 BEGIN_EVENT_TABLE(VSTEffectGUIDialog,wxDialog)
    EVT_BUTTON( wxID_OK, VSTEffectGUIDialog::OnOk )
    EVT_BUTTON( wxID_CANCEL, VSTEffectGUIDialog::OnCancel )
-   EVT_BUTTON( PREVIEW_ID, VSTEffectGUIDialog::OnPreview )
+   EVT_BUTTON( ID_EFFECT_PREVIEW, VSTEffectGUIDialog::OnPreview )
    EVT_IDLE( VSTEffectGUIDialog::OnIdle )
 END_EVENT_TABLE()
 
@@ -84,28 +84,15 @@ VSTEffectGUIDialog::VSTEffectGUIDialog(wxWindow *parent, wxWindowID id,
 
    mainSizer->Add(rect->right-rect->left, rect->bottom-rect->top);
 
-   wxBoxSizer *hSizer = new wxBoxSizer(wxHORIZONTAL);
-
-   wxButton *preview =
-      new wxButton(this, PREVIEW_ID, mEffect->GetPreviewName());
-   hSizer->Add(preview, 0, wxALIGN_CENTRE|wxALL, 5);
-
-   hSizer->Add(20, 10); // horizontal spacer
-
-   wxButton *cancel = new wxButton(this, wxID_CANCEL, _("Cancel"));
-   hSizer->Add(cancel, 0, wxALIGN_CENTRE|wxALL, 5);
-
-   wxButton *ok = new wxButton(this, wxID_OK, _("OK"));
-   ok->SetDefault();
-   ok->SetFocus();
-   hSizer->Add(ok, 0, wxALIGN_CENTRE|wxALL, 5);
-
-   mainSizer->Add(hSizer, 0, wxALIGN_CENTRE|wxALIGN_CENTER_VERTICAL|wxALL, 5);
+   // Preview, OK, & Cancel buttons
+   mainSizer->Add(CreateStdButtonSizer(this, ePreviewButton|eCancelButton|eOkButton), 0, wxEXPAND);
 
    SetAutoLayout(true);
    SetSizer(mainSizer);
    mainSizer->Fit(this);
    mainSizer->SetSizeHints(this);
+
+   mEffect->SetDialog(this);
 }
 
 void VSTEffectGUIDialog::OnIdle(wxIdleEvent &event)
@@ -152,6 +139,11 @@ wxString VSTEffect::GetEffectName()
    return pluginName + "...";
 }
 
+wxString VSTEffect::GetEffectCategories()
+{
+   return std::set<wxString>();
+}
+
 wxString VSTEffect::GetEffectAction()
 {
    return "Performing VST Effect: \""+pluginName+"\"";
@@ -177,8 +169,7 @@ bool VSTEffect::Init()
             GetSamples((WaveTrack *)right, &rstart, &rlen);
             
             if (llen != rlen || ((WaveTrack *)left)->GetRate() != ((WaveTrack *)right)->GetRate()) {
-               wxMessageBox(_("Sorry, VST Effects cannot be performed on stereo tracks where "
-                            "the individual channels of the track do not match."));
+               wxMessageBox(_("Sorry, VST Effects cannot be performed on stereo tracks where the individual channels of the track do not match."));
                return false;
             }
          }
@@ -262,7 +253,10 @@ void VSTEffect::GetSamples(WaveTrack *track,
 
 bool VSTEffect::Process()
 {
-   TrackListIterator iter(mWaveTracks);
+   this->CopyInputWaveTracks(); // Set up m_pOutputWaveTracks.
+   bool bGoodResult = true;
+
+   TrackListIterator iter(m_pOutputWaveTracks);
    int count = 0;
    Track *left = iter.First();
    Track *right;
@@ -280,24 +274,26 @@ bool VSTEffect::Process()
       // Reset the effect
       callDispatcher(aEffect, effOpen, 0, 0, NULL, 0.0);
       
-      bool success = ProcessStereo(count,
+      bGoodResult = ProcessStereo(count,
                                    (WaveTrack *)left, (WaveTrack *)right,
                                    lstart, rstart, len);
 
-      if (!success)
-         return false;
+      if (!bGoodResult)
+         break;
    
       left = iter.Next();
       count++;
    }
    
-   return true;
+   this->ReplaceProcessedWaveTracks(bGoodResult); 
+   return bGoodResult;
 }
 
 bool VSTEffect::ProcessStereo(int count, WaveTrack *left, WaveTrack *right,
                               longSampleCount lstart,
                               longSampleCount rstart, sampleCount len)
 {
+   bool rc = true;
    if (mBlockSize == 0) {
       mBlockSize = left->GetMaxBlockSize() * 2;
 
@@ -353,16 +349,20 @@ bool VSTEffect::ProcessStereo(int count, WaveTrack *left, WaveTrack *right,
       rs += block;
       
       if (inputs > 1) {      
-         if (TrackGroupProgress(count, (ls-lstart)/(double)originalLen))
+         if (TrackGroupProgress(count, (ls-lstart)/(double)originalLen)) {
+            rc = false;
             break;
+         }
       }
       else {
-         if (TrackProgress(count, (ls-lstart)/(double)originalLen))
+         if (TrackProgress(count, (ls-lstart)/(double)originalLen)) {
+            rc = false;
             break;
+         }
       }
    }
 
-   return true;
+   return rc;
 }
 
 void VSTEffect::End()
@@ -450,7 +450,7 @@ VSTEffectDialog::VSTEffectDialog(wxWindow * parent,
                    vst->GetPreviewName(),
                    wxPoint(10, y), wxSize(80, 30));
    wxButton *cancel =
-       new wxButton(this, wxID_CANCEL, _("Cancel"), wxPoint(130, y), wxSize(80, 30));
+       new wxButton(this, wxID_CANCEL, _("&Cancel"), wxPoint(130, y), wxSize(80, 30));
    wxButton *ok =
        new wxButton(this, wxID_OK, _("OK"), wxPoint(220, y), wxSize(80, 30));
 
@@ -467,6 +467,7 @@ VSTEffectDialog::VSTEffectDialog(wxWindow * parent,
    Centre(wxBOTH | wxCENTER_FRAME);
 
    SetSize(size);
+   vst->SetDialog(this);
 }
 
 VSTEffectDialog::~VSTEffectDialog()
@@ -663,3 +664,15 @@ float VSTEffect::callGetParameter(AEffect * effect, long index)
 #endif // MACOSX
 
 #endif // USE_VST
+
+// Indentation settings for Vim and Emacs and unique identifier for Arch, a
+// version control system. Please do not modify past this point.
+//
+// Local Variables:
+// c-basic-offset: 3
+// indent-tabs-mode: nil
+// End:
+//
+// vim: et sts=3 sw=3
+// arch-tag: dae3d099-e1eb-494e-b8eb-8f0af7f674d2
+
