@@ -12,14 +12,16 @@
 #define __AUDACITY_EFFECT_NYQUIST__
 
 #include <wx/button.h>
+#include <wx/datetime.h>
 #include <wx/dialog.h>
 #include <wx/filename.h>
-#include <wx/datetime.h>
-#include <wx/stattext.h>
-#include <wx/slider.h>
-#include <wx/textctrl.h>
-#include <wx/sizer.h>
 #include <wx/intl.h>
+#include <wx/sizer.h>
+#include <wx/slider.h>
+#include <wx/stattext.h>
+#include <wx/textctrl.h>
+#include <wx/tokenzr.h>
+
 #include "../Effect.h"
 
 #include "nyx.h"
@@ -60,14 +62,33 @@ public:
       return mName;
    }
    
+   virtual std::set<wxString> GetEffectCategories() {
+      return mCategories;
+   }
+
+   virtual wxString GetEffectIdentifier() {
+      if (mInteractive) {
+         // Disabled for now...
+         return wxT("");
+      }
+      
+      wxStringTokenizer st(mName, wxT(" "));
+      wxString id;
+
+      // CamelCase the name
+      while (st.HasMoreTokens()) {
+         wxString tok = st.GetNextToken();
+
+         id += tok.Left(1).MakeUpper() + tok.Mid(1);
+      }
+
+      return id;
+   }
+   
    virtual wxString GetEffectAction() {
       return mAction;
    }
    
-   virtual int GetEffectFlags() {
-      return mFlags;
-   }
-
    virtual bool PromptUser();
    
    virtual bool Process();
@@ -75,15 +96,15 @@ public:
 private:
    
    int GetCallback(float *buffer, int channel,
-                   long start, long len);
+                   long start, long len, long totlen);
    int PutCallback(float *buffer, int channel,
-                   long start, long len);
+                   long start, long len, long totlen);
 
    static int StaticGetCallback(float *buffer, int channel,
-                                long start, long len,
+                                long start, long len, long totlen,
                                 void *userdata);
    static int StaticPutCallback(float *buffer, int channel,
-                                long start, long len,
+                                long start, long len, long totlen,
                                 void *userdata);
 
    bool       ProcessOne();
@@ -104,7 +125,6 @@ private:
    wxString         mName;
    wxString         mAction;
    wxString         mInfo;
-   int              mFlags;
    bool             mDebug;
    wxString         mDebugOutput;
 
@@ -112,18 +132,23 @@ private:
 
    int              mCurNumChannels;
    WaveTrack       *mCurTrack[2];
-   longSampleCount  mCurStart[2];
+   sampleCount      mCurStart[2];
    sampleCount      mCurLen;
    double           mOutputTime;
    int              mCount;
-   double           mProgress;
+   double           mProgressIn;
+   double           mProgressOut;
+   double           mProgressTot;
+   double           mScale;
 
    samplePtr        mCurBuffer[2];
-   longSampleCount  mCurBufferStart[2];
+   sampleCount      mCurBufferStart[2];
    sampleCount      mCurBufferLen[2];
 
    WaveTrack       *mOutputTrack[2];
    
+   std::set<wxString> mCategories;
+   
 };
 
 class NyquistDialog:public wxDialog {
@@ -140,6 +165,7 @@ class NyquistDialog:public wxDialog {
 
    void OnText(wxCommandEvent & event);
    void OnSlider(wxCommandEvent & event);
+   void OnChoice( wxCommandEvent &event );
    void OnOk(wxCommandEvent & event);
    void OnDebug(wxCommandEvent & event);
    void OnCancel(wxCommandEvent & event);
@@ -186,3 +212,15 @@ class NyquistOutputDialog:public wxDialog {
 
 #endif
 
+
+// Indentation settings for Vim and Emacs and unique identifier for Arch, a
+// version control system. Please do not modify past this point.
+//
+// Local Variables:
+// c-basic-offset: 3
+// indent-tabs-mode: nil
+// End:
+//
+// vim: et sts=3 sw=3
+// arch-tag: 869e264a-73b8-4c7b-ad9a-c66168e57ee5
+
